import { Column, Entity, PrimaryGeneratedColumn, Unique } from 'typeorm';
import bcrypt from 'bcrypt';

@Entity('users')
@Unique(['username'])
export class User {
  @PrimaryGeneratedColumn()
  id!: number;

  @Column()
  username!: string;

  @Column()
  password?: string;

  @Column()
  displayName!: string;

  @Column()
  token!: string;

  async comparePassword(password: string): Promise<boolean> {
    if (this.password) return await bcrypt.compare(password, this.password);
    return false;
  }

  generateToken() {
    this.token = crypto.randomUUID();
  }
}
