import { Repository } from 'typeorm';
import { User } from '@/entities/user.entity';
import { appDataSource } from '@/config/dataSource';
import { SignInUserDto } from '@/dto/sign-in-user.dto';
import { RegisterUserDto } from '@/dto/register-user.dto';
import { IUser } from '@/interfaces/IUser.interface';
import bcrypt from 'bcrypt';

const SALT_WORK_FACTORY = 10;
export class UserRepository extends Repository<User> {
  constructor() {
    super(User, appDataSource.createEntityManager());
  }

  async signIn(signInUserDto: SignInUserDto): Promise<User> {
    const user = await this.findOne({
      select: ['username', 'displayName', 'id', 'password'],
      where: { username: signInUserDto.username },
    });

    if (!user) throw new Error('User with this username not exists');

    const isMatch = await user.comparePassword(signInUserDto.password);
    if (!isMatch) throw new Error('Password is wrong');

    user.generateToken();
    const userWithToken = await this.save(user);
    delete userWithToken.password;
    return userWithToken;
  }

  async register(registerUserDto: RegisterUserDto): Promise<IUser> {
    const salt = await bcrypt.genSalt(SALT_WORK_FACTORY);
    const userData = await this.create(registerUserDto);
    userData.password = await bcrypt.hash(registerUserDto.password, salt);
    userData.generateToken();
    const user = await this.save(userData);
    delete user.password;

    return user;
  }

  async getUserByToken(token: string): Promise<IUser | null> {
    return await this.findOneBy({ token });
  }
}
